`default_nettype none

module coilcounter(clk50mhz, KEY[3:0], HEX0, HEX1, HEX2, HEX3, GPIO_0);
input			clk50mhz;
input [3:0] 	KEY;
output [6:0] 	HEX0;
output [6:0] 	HEX1;
output [6:0] 	HEX2;
output [6:0] 	HEX3;
input [12:0] 	GPIO_0;

//    ,  
wire clk = clk50mhz;                                //  
wire reset = ~KEY[0];                               // ,  1
wire reed = GPIO_0[9];                              //   

reg [15:0] turnscount;                              //  
reg        sensorsampled;                           //    

reg [17:0] slooow;                                  //  
wire       slow_enable = slooow == 0;               //   

//      
//     slow_enable
always @(posedge clk) begin: _slow
    slooow <= slooow + 1'b1;
end

//        
//  
always @(posedge clk) begin: _sample
    if (slow_enable) sensorsampled <= reed;
end

wire plus_one = ~sensorsampled && reed;             //   

`ifdef hex //   , 

always @(posedge clk) begin
    if (reset) 
        turnscount <= 0;
    else 
        if (slow_enable) begin
            if (plus_one) turnscount <= turnscount + 1'b1;
        end
end

SEG7_LUT_4 seg7display(HEX0, HEX1, HEX2, HEX3, turnscount);

`else //  ,    

wire [15:0] bcd_turns;                              //    
                                                    //  - 
wire        cy0,cy1,cy2;                            //   

//     ,   plus_one & slow_enable, 
//     3:0  bcd_turns,    cy0
decade_counter ctr0(.clk(clk), .ena(plus_one & slow_enable), .reset(reset),
                    .value(bcd_turns[3:0]), .c(cy0));
                    
//    ,      
decade_counter ctr1(.clk(clk), .ena(plus_one & slow_enable & cy0), .reset(reset),
                    .value(bcd_turns[7:4]), .c(cy1));
// ...                    
decade_counter ctr2(.clk(clk), .ena(plus_one & slow_enable & cy0 & cy1), .reset(reset),
                    .value(bcd_turns[11:8]), .c(cy2));
// ...                    
decade_counter ctr3(.clk(clk), .ena(plus_one & slow_enable & cy0 & cy1 & cy2), .reset(reset),
                    .value(bcd_turns[15:12]));

SEG7_LUT_4 seg7display(HEX0, HEX1, HEX2, HEX3, bcd_turns);

`endif

endmodule

//     
module decade_counter(input clk,
              input ena,
              input reset,
              output reg[3:0] value,
              output c);

assign c = value == 9;

always @(posedge clk) 
    if (reset) 
        value <= 4'b0;
    else if (ena) 
        value <= value == 9 ? 0 : (value + 1'b1);
    
endmodule

